from pygame import *
import pygame
import time
import math
import random
import map
import actor
import playeractor
import controls
import sound
import os
import unpack
from vector import *
import entities


class Engine:    
    def __init__(self,mapFile,bgmFile=None,startWithPlayer=True,surface=None):
        self.finishedLevel1=False
        self.finishedLevel2=False
        #if we're passed a surface, it's assumed that pygame is already initialized
        if not surface:
            self.screenSize=(640,480)
            self.screenWidth= self.screenSize[0]
            self.screenHeight= self.screenSize[1]
            self.startPygame()
        else:
            pygame.init()
            self.screen=surface
            self.screenSize=(surface.get_width(),surface.get_height())
            self.screenWidth= self.screenSize[0]
            self.screenHeight= self.screenSize[1]
        
        self.debug=False

        #self.enemySprites.add(entities.SpikeBall(self))
        self.initEngine()

        
        if startWithPlayer:
            self.playerSprite=playeractor.PlayerActor(self,position=(3*32,27*32))
            self.playerSpriteGroup.add(self.playerSprite)
        else:
            self.playerSprite=None
            self.doFrame=self.renderFrame
            
        self.startLevel1()


        self.tick=0
        self.scrollToleranceX=70
        self.scrollToleranceY=50
        


    def startLevel1(self):
        self.enemySprites=Group()
        
        img=pygame.image.load('media/moon.png').convert()
        #self.addBg(img,True,True,(.2,0))
        self.addBg(img,False,False,(0,0))    
        img=pygame.image.load('media/clouds2.png').convert_alpha()
        #self.addBg(img,True,True,(.2,0))
        self.addBg(img,True,False,(.4,0))
        
        self.insertMapEntities('media/level1.txt')
        print "bing"
        self.sound.playBGM('media/bgm/Level_Three.ogg')
        self.allGroups=self.enemySprites,self.playerBullets,self.bgSprites,self.fgSprites
        tilemap={}
        tilemap[1]='media/tiles/tile000.png'
        tilemap[6]='media/tiles/tile000.png'
        tilemap[2]='media/tiles/dirt.png'
        tilemap[3]='media/tiles/brick.png'
        tilemap[246]='media/tiles/slime.png'
        self.map=map.Map('media/level1.png','',32,tilemap)
        self.finishedLevel1=False
       
    def startLevel2(self):
        self.bgs=[]
        self.bgPos=[]
        self.enemySprites=Group()
        img=pygame.image.load('media/moon.png').convert()

        self.playerSprite.position=(7*32,7*32)
        self.playerSprite.spawnPosition=(7*32,7*32)

        self.sound.playBGM('media/bgm/Level_Two.ogg')
        self.allGroups=self.enemySprites,self.playerBullets,self.bgSprites,self.fgSprites
        tilemap={}
        tilemap[1]='media/tiles/tile000.png'
        tilemap[6]='media/tiles/tile000.png'
        tilemap[2]='media/tiles/dirt.png'
        tilemap[3]='media/tiles/brick.png'
        tilemap[246]='media/tiles/slime.png'
        self.map=map.Map('media/level2.png','',32,tilemap)
        self.finishedLevel2=False
        img=pygame.image.load('media/moon.png').convert()
   
        self.addBg(img,False,False,(0,0))    
        img=pygame.image.load('media/clouds2.png').convert_alpha()
       
        self.addBg(img,True,False,(.4,0))
        try:
            self.insertMapEntities('media/level2.txt')
        except:
            pass

    def initEngine(self):
        self.doFrame=self.inGameDoFrame
        self.playerBullets=Group()
        self.bgSprites=Group()
        self.playerSpriteGroup=Group()
        self.fgSprites=Group()
        self.level1DoneImage=pygame.image.load('media/level1done.png').convert()
        self.scrollPosition=(0,0)
        self.sound=sound.Sound()
        self.controls=controls.Controls()
        self.bgs=[]
        self.bgPos=[]
        pygame.font.init()
        self.font=pygame.font.Font(pygame.font.get_default_font(),16)
        self.healthText=self.font.render("Health",True,(128,128,255))
        
    def addBg(self,img,tileH,tileV,scrollScale):
        self.bgs.append((img,scrollScale,tileH,tileV))
        self.bgPos.append((0,0))
    def renderBg(self):
        idx=0
        for bg in self.bgs:
            img,scrollscale,tileH,tileV=bg
            ssx,ssy=scrollscale
            sx,sy=self.scrollPosition
            imwid=img.get_width()
            imhei=img.get_height()
            self.screen.blit(img,((-sx*ssx)%imwid,(-sy*ssy)%imhei))
            if tileH:
                self.screen.blit(img,(-imwid+(-sx*ssx)%imwid,(-sy*ssy)%imhei))
                self.screen.blit(img,((-sx*ssx)%imwid,-imhei+(-sy*ssy)%imhei))
            if tileV:
                self.screen.blit(img,(imwid+(-sx*ssx)%imwid,(-sy*ssy)%imhei))
                self.screen.blit(img,((-sx*ssx)%imwid,imhei+(-sy*ssy)%imhei))
            
            if tileH and tileV:
                self.screen.blit(img,(imwid+(-sx*ssx)%imwid,imhei+(-sy*ssy)%imhei))
                self.screen.blit(img,(-imwid+(-sx*ssx)%imwid,imhei+(-sy*ssy)%imhei))
                self.screen.blit(img,(imwid+(-sx*ssx)%imwid,-imhei+(-sy*ssy)%imhei))
                self.screen.blit(img,(-imwid+(-sx*ssx)%imwid,-imhei+(-sy*ssy)%imhei))
            idx+=1
    def startPygame(self):
        pygame.init()
        self.screen=pygame.display.set_mode(self.screenSize)
        pygame.display.set_caption("The Magic Thief")
        self.isFullscreen=False
        
    def setFullscreen(self,val):
        if self.isFullscreen:
            self.screen=pygame.display.set_mode(self.screenSize)
            pygame.mouse.set_visible(True)
            self.isFullscreen=False
        else:
            screen=pygame.display.set_mode(self.screenSize,pygame.FULLSCREEN)
            pygame.mouse.set_visible(False)
            self.isFullscreen=True
            
    def insertEntity(self,entName,entPos,entArgs):
        try:
            print "insertEntity", entName, entPos, entArgs
            exec('ent=entities.%s(self,position=entPos,%s)'%(entName,entArgs))
        except:
            print "bad kw arg!"
        self.enemySprites.add(ent)

    def insertMapEntities(self,file_name):
        entities = self.loadMapEntities(file_name)
        for ent in entities:
            name,pos,args = ent
            self.insertEntity(name,pos,args)
            
    def loadMapEntities(self,file_name):
        try:
            mapinfo = list()
            fh = open(file_name, "r")
            lines = fh.readlines()
            for line in lines:
                bundle = line.split(":")
                if len(bundle) == 1:
                    map_image_name = bundle[0]
                else:
                    rname, pos, kwargs = bundle
                    exec('posn='+pos)
                    mapinfo.append((rname,posn,kwargs))
            fh.close()
            
        except:
            mapinfo = list()
            print "Error: No Map Error"
        return mapinfo
        
    def runFrame(self,event):
        self.tick+=1
        self.controls.update(event)
        self.screen.fill((0,0,0))
        self.baseDoFrame()
        self.doFrame()
        pygame.display.flip()
        
    def baseDoFrame(self):
        pass

    def nullDoFrame(self):
        pass

    def inGameDoFrame(self):
        self.renderBg()
        x,y=self.scrollPosition
        #print(self.scrollPosition)
        
        self.scrollAroundPlayer()
        #self.scrollPosition=subv(self.playerSprite.position,(self.screenWidth/2,self.screenHeight/2))
        self.map.drawMap(self.screen,self.scrollPosition)
        self.playerSpriteGroup.think()
        self.playerSpriteGroup.draw(self.screen)
        self.enemySprites.think()
        self.enemySprites.draw(self.screen)
        self.playerSpriteGroup.collide(self.enemySprites)
        self.drawHud()
        
        if self.finishedLevel1:
            self.sound.stopBGM()
            self.doFrame=self.level1Finished
            self.startLevel2Time=self.tick+120
    
    def drawHud(self):
        self.screen.blit(self.healthText,(0,0))
        pygame.draw.rect(self.screen,(0,0,255),(5,25,self.playerSprite.health*2,10))
        self.screen.blit(\
        self.font.render('Lives:%d'%self.playerSprite.lives,True,(128,128,255)),\
        (580,25))
    def  level1Finished(self):
        self.screen.blit(self.level1DoneImage,(0,0))
        if self.tick == self.startLevel2Time:
            self.startLevel2()
            self.doFrame=self.inGameDoFrame
            
        
    def scrollAroundPlayer(self):
        playerCenteredx,playerCenteredy=\
        subv(self.playerSprite.position,\
        (self.screenWidth/2,self.screenHeight*.600))
        sx,sy=self.scrollPosition
        diffx=playerCenteredx-sx
        diffy=playerCenteredy-sy
        if fabs(diffx) > self.scrollToleranceX:
            sx+= sign(diffx)*(fabs(diffx)-self.scrollToleranceX)
        if fabs(diffy) > self.scrollToleranceY:
            sy+= sign(diffy)*(fabs(diffy)-self.scrollToleranceY)
        self.scrollPosition=(sx,sy)
        
    def renderFrame(self):
        x,y=self.scrollPosition
        #print(self.scrollPosition)
        #self.playerSprite.think()
        #self.scrollPosition=subv(self.playerSprite.position,(self.screenWidth/2,self.screenHeight/2))
        #print self.scrollPosition
        for actor in self.enemySprites.actors:
            actor.draw(self.screen)
        #self.screen.blit(self.map.scaledMap,(100,1000))
        self.map.drawMap(self.screen,self.scrollPosition)
        #self.playerSprite.draw(self.screen)
    
    def mapToScreen(self, mcoord):
        return subv(mcoord,self.scrollPosition)
    
    def screenToMap(self, scoord):
        return addv(scoord,self.scrollPosition)
    
class Group:
    def __init__(self):
        self.actors=[]
        
    def add(self,actr):
        self.actors.append(actr)
        actr.groups.append(self)
    
    def remove(self,actr):
        self.actors.remove(actr)
        actr.groups.remove(self)
    
    def clear(self):
        self.actors.clear()
    
    def think(self):
        for a in self.actors:
            a.think()
    def draw(self,surf):
        for a in self.actors:
            a.draw(surf)
    def collide(self,other):
        """checks for collisions between this group and the other. 
        onCollide is called on objects that collide between the two groups"""
        if len(self.actors)<len(other.actors):
            a=self.actors
            b=other.actors
        else:
            b=self.actors
            a=other.actors
        for act in a:
            collvec=act.screenbounds.collidelistall([arg.screenbounds for arg in b])
            hitters=[b[idx] for idx in collvec]
            [(act.onCollide(h),h.onCollide(act)) for h in hitters]
