import actor
from vector import *
import pygame
import grapple
import unpack

walkSpeed=3.8
maxFallSpeed=16
g=0.4
jumpSpeed=8
airControl=0.09
playerWidth=14
playerHeight=80
ropeK=320
topSolid=[1,2,4,5,6]
bottomSolid=[1,2,4,5,6]
horizontalSolid=[1,2,4,5,6]
frameDelay=5
maxGrapLen=250
grapXVel=13
grapYVel=12
#the point relative to player origin 
#that the grappling hook shoots from
grappleOffset=(0,-60)
damageDelay=10
#yeah, we don't have enums. may not even need these, if we're clever
falling=0 
standing=1
walking=2
landing=3 # engaged after hitting ground, ends after land animation
jumping=4 #turns into falling at apex?
clinging=5

left=0
right=1

startLives=3

class PlayerActor(actor.Actor):
    def __init__(self,engine,**kw):
        actor.Actor.__init__(self,engine,**kw)
        self.bounds=pygame.Rect(-playerWidth,-playerHeight,playerWidth*2,playerHeight)
        
        self.spawnPosition=self.position
        anims=unpack.unpack('Resources','resources.tar')
        self.respawn()
        self.frameMap=[{},{}]
        self.frameMap[right][walking]=anims['walk']
        self.frameMap[right][standing]=anims['stand']
        self.frameMap[right][falling]=anims['falling']
        self.frameMap[right][clinging]=anims['cling']
        self.clingDir=self.direction
        self.image=self.frameMap[right][walking][0]
        for flist in self.frameMap[right]:
            self.frameMap[left][flist]=self.flipFrames(self.frameMap[right][flist])
        
        self.nextDamage=0
        self.lives=startLives
        self.dead=False

    def draw(self,surface):
        actor.Actor.draw(self,surface)
        if self.grappling or self.grappleLaunched:
            pygame.draw.line(surface,(255,0,0),addv(self.drawPosition,grappleOffset),\
            subv(self.grappleAnchor,self.engine.scrollPosition))
    
    def flipFrames(self,flist):
        return [(pygame.transform.flip(f[0],True,False),\
        (f[0].get_width()-f[1][0],f[1][1])) for f in flist]
        
    def advanceAnim(self):
        
        frameList=self.frameMap[self.direction].get(self.state)
        if frameList:
            self.curFrame+=1.0/frameDelay
            if self.curFrame>=len(frameList):
                self.curFrame=0
            self.image=frameList[int(self.curFrame)]
    
    def checkHealth(self):
        px,py=self.position
        if px>self.engine.map.mapWidth*self.engine.map.blockSize or\
            py>self.engine.map.mapHeight*self.engine.map.blockSize:
            self.health=0
        
        if self.health<=0:
            self.lives-=1
            if self.lives>=0:
                self.respawn()
            else:
                self.dead=True
    
    def respawn(self):
        self.health=100
        self.fallVelocity=0
        self.state=falling
        self.walkVelocity=0
        self.curFrame=0
        
        self.direction=1
        self.grappleAnchor=(0,0)
        self.grappling=False
        self.grappleLaunched=False
        self.position=self.spawnPosition
        
    def think(self):
        if self.dead:
            return
        actor.Actor.think(self)
        self.checkHealth()
        #convenience vars
        theMap=self.engine.map
        ctrls=self.engine.controls
        sound=self.engine.sound
        justJumped=False
        if (self.state == falling or self.state == jumping) and not self.grappling:
            #print "falling!"
            #if underfoot == 0: #we're falling!
            self.fallVelocity+=g
            if self.fallVelocity>maxFallSpeed:
                self.fallVelocity = maxFallSpeed
            if ctrls.left and self.walkVelocity>-walkSpeed:
                self.walkVelocity+=-walkSpeed*airControl
                #self.direction=0
            if ctrls.right and self.walkVelocity<walkSpeed:
                self.walkVelocity+=walkSpeed*airControl
                #self.direction=1
        if (self.state == falling or self.state == jumping) and self.grappling:
            if ctrls.left and self.walkVelocity>-walkSpeed:
                self.walkVelocity+=-walkSpeed*airControl
                
            if ctrls.right and self.walkVelocity<walkSpeed:
                self.walkVelocity+=walkSpeed*airControl

        if self.state == standing or self.state == walking:
            self.walkVelocity=0
            self.state=standing
            if ctrls.left:
                if not theMap.getBlock(addv(self.position,(-playerWidth,-1))) in horizontalSolid:
                    self.walkVelocity=-walkSpeed
                    self.state = walking
                    self.direction=0
            if ctrls.right:
                if not theMap.getBlock(addv(self.position,(playerWidth,-1))) in horizontalSolid:
                    self.walkVelocity=walkSpeed
                    self.state = walking
                    self.direction=1
        #handle the jump button
        if ctrls.ahit:
            if self.state == standing or self.state == walking:                
                justJumped=True
                sound.play('jump')
                self.fallVelocity=-jumpSpeed
                self.state=falling
                if ctrls.right:
                    self.walkVelocity=walkSpeed
                    self.direction=right
                if ctrls.left:
                    self.walkVelocity=-walkSpeed
                    self.direction=left
            #jump off from a cling!
            elif self.state==clinging:
                sound.play('jump')
                self.state=falling
                if ctrls.right and self.clingDir==left:
                    self.fallVelocity=-jumpSpeed
                    self.walkVelocity=walkSpeed
                    self.direction=right
                elif ctrls.left and self.clingDir==right:
                    self.fallVelocity=-jumpSpeed
                    self.walkVelocity=-walkSpeed
                    self.direction=left
                else:
                    self.walkVelocity=-(self.direction*2 -1)*walkSpeed*0.3
        
        if ctrls.bhit and self.state == falling and not self.grappling:
            #Launch our grappling hook!
            self.grappleAnchor=addv(self.position,grappleOffset)
            self.graplen=0
            self.grappleLaunched=True
            self.grapdir=1
            if ctrls.down:
                self.grapdir=-1

        
        if ctrls.b and self.state==falling:
            if self.grappleLaunched:
                #print "grappling!"
 
                self.grappleAnchor=addv(self.grappleAnchor,(grapXVel*(self.direction*2-1),-grapYVel*self.grapdir))
                sound.play('grapple')
                if distv(addv(self.position,grappleOffset),self.grappleAnchor) > maxGrapLen:
                    self.grappleLaunched=False
                    
                elif theMap.getBlock(self.grappleAnchor) in bottomSolid:
                    #grapple has landed!
                    sound.play('grapplehit')
                    self.grappling =True
                    self.grappleLaunched=False
                    self.graplen=distv(addv(self.position,grappleOffset),self.grappleAnchor)
                    self.fallVelocity=0
                    self.walkVelocity*=1.5
                    self.grappling=True

                    #play sound
                #print vel
            
            if self.grappling:
                #print "grappling!"
                tempPos,vel=\
                grapple.solve(ropeK,0,g*0,self.graplen,self.grappleAnchor,addv(self.position,grappleOffset)\
                ,(self.walkVelocity*60,self.fallVelocity*60),1/60.0)
                self.walkVelocity,self.fallVelocity=mulv(vel,1/60.0)
                self.fallVelocity+=g
                self.position=subv(tempPos,grappleOffset)
                #print vel
            else:
                self.grappling=False
        #lastly, check for wall bounds
        underfoot=theMap.getBlock(addv(self.position,(0,self.fallVelocity)))
        if not underfoot in bottomSolid and self.state != clinging:
            self.state=falling
        
        colLow=theMap.getBlock(addv(self.position,(sign(self.walkVelocity)*playerWidth,-1)))
        colMid=theMap.getBlock(addv(self.position,(sign(self.walkVelocity)*playerWidth,-30)))
        colHigh=theMap.getBlock(addv(self.position,(sign(self.walkVelocity)*playerWidth,-56)))
        colTop=theMap.getBlock(addv(self.position,(0,-playerHeight)))
        
        horizontalCollision=max(max(colLow,colHigh),colMid)
        if horizontalCollision in horizontalSolid:
            self.grappling=False

        #clip and apply horizontal movement!
        if self.walkVelocity:
            if horizontalCollision in horizontalSolid: #hitawall!
                self.walkVelocity=0
                #ok, we've hit a wall, let's check our clinging dynamics
                if self.state==falling and not justJumped:
                    if ctrls.ahit and colMid in horizontalSolid\
                     and ((self.direction==right and ctrls.right)\
                     or (self.direction==left and ctrls.left)):
                        self.state=clinging
                        self.clingDir=self.direction
            else:
                if not self.grappling:
                    self.position=addv(self.position,(self.walkVelocity,0))
        #clip and apply vertical movement!
        if self.state == falling:
            if not ctrls.b:
                self.grappleLaunched=False
                self.grappling=False
            if  underfoot in bottomSolid and self.state == falling: #landed
                    sound.play('land')
                    self.state=standing
                    self.fallVelocity=0
                    self.position=theMap.moveYToBlockEdge(self.position)
                    self.grappleLaunched=False
                    self.grappling=False
            elif colTop in topSolid:
                self.position=addv(self.position,subv(self.position,theMap.moveYToBlockEdge(self.position)))
                self.fallVelocity*=-1
            if not self.grappling:
                self.position=addv(self.position,(0,self.fallVelocity))
        self.advanceAnim()
    
    def onCollide(self,other):
        if other.enemy and self.tick>self.nextDamage:
            self.walkVelocity=-(self.direction*2-1)*5
            self.fallVelocity=-3
            self.state=falling
            self.grappling=False
            self.grappleLaunched=False
            self.health-=other.damage
            self.nextDamage=self.tick+damageDelay
            self.engine.sound.play('ouch')
            print self.health
