#!/usr/bin/env python
import os, sys, pygame, tarfile
from pygame.locals import *

import sys

sys.path.insert(0, "..")
sys.path.append("../game/")

from pgu import gui
from vector import *
pygame.init()
pygame.font.init()

class NewDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("New Resource")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("Resource Name"),align=0,colspan=3)
        layout.tr()
        self.image_name = gui.Input(name="image_name")
        layout.td(self.file_name,colspan=3)
        layout.tr()
        ok_button = gui.Button("Okay")
        ok_button.connect(gui.CLICK,self.close,None)
        layout.td(ok_button)
        gui.Dialog.__init__(self,title,layout)

class SaveDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("Save Resource As")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("File Name:"))
        self.file_rsc = gui.Input(name="output_file")
        layout.td(self.file_rsc)
        b_button = gui.Button("Browse")
        b_button.connect(gui.CLICK, self.open_browser, None)
        layout.td(b_button)
        s_button = gui.Button("Ok")
        s_button.connect(gui.CLICK, self.send, gui.CHANGE)
        layout.td(s_button)
        gui.Dialog.__init__(self,title,layout)
    
    def open_browser(self, value):
        browser = gui.FileDialog()
        browser.connect(gui.CHANGE, self.close_browser, browser)
        browser.open()
    
    def close_browser(self, rsc):
        if rsc.value: self.file_rsc.value = rsc.value
        
class OpenDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("Open Resource")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("File Name:"))
        self.file_rsc = gui.Input(name="input_file")
        layout.td(self.file_rsc)
        b_button = gui.Button("Browse")
        b_button.connect(gui.CLICK, self.open_browser, None)
        layout.td(b_button)
        s_button = gui.Button("Ok")
        s_button.connect(gui.CLICK, self.send, gui.CHANGE)
        layout.td(s_button)
        gui.Dialog.__init__(self,title,layout)
    
    def open_browser(self, value):
        browser = gui.FileDialog()
        browser.connect(gui.CHANGE, self.close_browser, browser)
        browser.open()
    
    def close_browser(self, rsc):
        if rsc.value: self.file_rsc.value = rsc.value
        
class ExitDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("Exit")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.add(gui.Label("Are you sure you want to Exit?",colspan=2))
        layout.tr()
        layout.add(gui.Label("(All unsaved changes will be lost!)",colspan=2))
        layout.tr()
        e_button = gui.Button("Exit")
        e_button.connect(gui.CLICK,self.send,gui.QUIT)
        layout.td(e_button)
        
        c_button = gui.Button("Cancel")
        c_button.connect(gui.CLICK,self.close,None)
        layout.td(c_button)
        
        gui.Dialog.__init__(self,title,layout)
        
class FrameAdder(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("Add Frame Resource")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("File Name:"))
        self.file_rsc = gui.Input(name="file_name")
        layout.td(self.file_rsc)
        b_button = gui.Button("Browse")
        b_button.connect(gui.CLICK, self.open_browser, None)
        layout.td(b_button)
        s_button = gui.Button("Ok")
        s_button.connect(gui.CLICK, self.send, gui.CHANGE)
        layout.td(s_button)
        gui.Dialog.__init__(self,title,layout)
    
    def open_browser(self, value):
        browser = gui.FileDialog()
        browser.connect(gui.CHANGE, self.close_browser, browser)
        browser.open()
    
    def close_browser(self, rsc):
        if rsc.value: self.file_rsc.value = rsc.value
    
class FrameEditor(gui.Widget):
    def __init__(self, **params):
        gui.Widget.__init__(self, **params)
        self.position = None
        self.surface = None
        self.image_loaded = False
        self.bg_color = pygame.Color('#FFFFFF')
        self.vect = (0,0)
        self.is_offset = False
        self.colors = {'tc_point':(0,255,255),'o_point':(255,0,0), 'h_point':(0,255,0),'w_point':(0,0,255)}
        self.pdict = dict()

    def init(self, **params):
        self.width = 200
        self.height = 200
        self.__dict__.update(params)
        self.surface = pygame.Surface((self.width, self.height), pygame.SRCALPHA, 32)
        self.surface.fill(self.bg_color)
        self.repaint
        
    def paint(self,surface):
        surface.blit(self.surface,(0,0))
        
    def event(self, ev):
        if not self.surface or not self.image_loaded: return
        
        if app.mode:
            if self.pdict.has_key(app.mode):
                self.pdict.pop(app.mode)
                self.update_display()
            if ev.type == gui.MOUSEBUTTONDOWN:  
                app.fb_list[self.position][1][app.mode] = ev.pos
                self.pdict[app.mode] = ev.pos
                self.update_display()
                app.mode = None

    def update_display(self):
        if not self.surface: return
        if self.pdict.has_key('o_point') and self.pdict.has_key('tc_point'):
            tx, ty = self.pdict['tc_point']
            print "tc_point: ", tx, ty
            ox, oy = self.pdict['o_point']
            if tx != ox and ty != oy:
                self.vect = subv(self.pdict['tc_point'], self.pdict['o_point'])
                self.is_offset = True
            anc_ref = (self.surface.get_width()/2, self.surface.get_height()/1.5)
            x, y = addv(anc_ref, self.vect)
        else:
            x = self.surface.get_width()/6
            y = self.surface.get_height()/6
            self.pdict['tc_point'] = (x, y)
            self.vect = (0,0)
            self.is_offset = False

        self.surface.fill(self.bg_color)
        image = pygame.image.load(app.fb_list[self.position][1]['full_name'])
        self.surface.blit(image, (x, y))
        #self.surface.fill(,(x,y,4,4))
        vdict = self.pdict
        for k in vdict:
            print "vdict itorate"
            if self.is_offset:
                temp_x, temp_y = subv(vdict[k], self.vect)
            else:
                temp_x, temp_y = vdict[k]
            self.surface.fill(self.colors[k],(temp_x,temp_y,4,4))
        self.repaint()

    def edit(self, pos):
        self.position = pos
        self.pdict = dict()
        if app.fb_list[self.position][1].has_key('o_point'):
            self.pdict['o_point'] = app.fb_list[self.position][1]['o_point']
        if app.fb_list[self.position][1].has_key('h_point'):
            self.pdict['h_point'] = app.fb_list[self.position][1]['h_point']
        if app.fb_list[self.position][1].has_key('w_point'):
            self.pdict['w_point'] = app.fb_list[self.position][1]['w_point']

        self.update_display()
        self.image_loaded = True
    
class App(gui.Desktop):
    def __init__(self, **params):
        gui.Desktop.__init__(self,**params)
        self.connect(gui.QUIT,self.quit,None)
        
        gc = gui.Container(width=640, height=480)
        self.fname = 'unnamed.rsc'
        self.new_dialog = NewDialog()
        self.new_dialog.connect(gui.CHANGE,self.do_new,None)
        self.open_dialog = OpenDialog()
        self.open_dialog.connect(gui.CHANGE,self.do_open,None)
        self.save_dialog = SaveDialog()
        self.save_dialog.connect(gui.CHANGE, self.do_save,None)
        self.exit_dialog = ExitDialog()
        self.exit_dialog.connect(QUIT,self.quit,None)
        
        self.menus = gui.Menus([
            ('File/New',self.new_dialog.open,None),
            ('File/Open',self.open_dialog.open,None),
            ('File/Save',self.do_save,None),
            ('File/Save As',self.save_dialog.open,None),
            ('File/Exit',self.exit_dialog.open,None),
            ('About/Help',self.new_dialog.open,None),
            ('About/About',self.new_dialog.open,None)
            ])
        gc.add(self.menus,0,0)
        self.menus.rect.w,self.menus.rect.h = self.menus.resize()
        
        # Creates Main Working Area
        self.frame_editor = FrameEditor(width=300, height=300, style={'border':1})
        self.frame_editor.init(width=300, height=300)
        
        # Side Tools
        self.mode = None
        self.ls = gui.Table(width=130)
        self.ls.tr()
        self.ls.td(gui.Label("Frame Tools:"))
        
        # Add Origin Point
        self.ls.tr()
        op = gui.Button("Origin",width=130,height=25,color=(255,0,0))
        mode = "o_point"
        op.connect(gui.CLICK,self.set_mode,mode)
        self.ls.td(op)
        
        # Add Bounding Box
        self.ls.tr()
        bb = gui.Button("Bounding Box",width=130,height=25)
        #mode = "box"
        #bb.connect(gui.CLICK,frame_editor,mode)
        self.ls.td(bb)
        
        # Add Head Anchor
        self.ls.tr()
        hb = gui.Button("Head Anchor",width=130,height=25)
        mode = "h_point"
        hb.connect(gui.CLICK,self.set_mode,mode)
        self.ls.td(hb)
        
        # Add Weapon Anchor
        self.ls.tr()
        wb = gui.Button("Weapon Anchor",width=130,height=25)
        mode = "w_point"
        wb.connect(gui.CLICK,self.set_mode,mode)
        self.ls.td(wb)
        
        self.tools = gui.ScrollArea(self.ls,150,320,hscrollbar=False)
        gc.add(self.tools, 10, self.menus.rect.h)
        
        # Frame Picker
        self.rsc_data = []
        self.fb_list = []
        self.position = 0
        self.bt_wrapper = gui.Table()
        self.bt_wrapper.tr()
        bt_buttons = gui.Table(width=120, heigh=100)
        bt_buttons.tr()
        bt_buttons.td(gui.Label("Frames:"))
        bt_buttons.tr()
        self.add_frame = FrameAdder()
        self.add_frame.connect(gui.CHANGE, self.add_frame_rsc, None)
        fadd = gui.Button("Add", width=100, height=30)
        fadd.connect(gui.CLICK,self.add_frame.open,None)
        bt_buttons.td(fadd)
        fdel = gui.Button("Delete", width=100, height=30)
        fdel.connect(gui.CLICK,self.del_frame_rsc,None)
        bt_buttons.tr()
        bt_buttons.td(fdel)
        self.bt_wrapper.td(bt_buttons)
        self.bt = gui.Table()
        self.bt.tr()
        
        frames = gui.ScrollArea(self.bt,gc.rect.w-20,120)
        self.bt_wrapper.td(frames)
        gc.add(self.bt_wrapper, 10, self.menus.rect.h+self.tools.rect.h+5)
        
        # Add Main Drawing Area to Container
        gc.add(self.frame_editor, self.tools.rect.w+30, self.bt_wrapper.rect.h+self.menus.rect.h)
        
        self.frame_editor.rect.h, self.frame_editor.rect.w = self.frame_editor.resize()
        self.widget = gc
    
    def do_new(self, value):
        pass
    
    def do_open(self, value):
        self.open_dialog.close()
        self.file_name = self.open_dialog.value['input_file'].value
        print "File Name: " + self.file_name
        
        short_fn = os.path.split(self.file_name)[1]
        
        no_ext = os.path.splitext(short_fn)[0]
        
        rsc_name = os.path.normpath(no_ext+ '.txt')
        print rsc_name
        fh = tarfile.open(self.file_name, "r")
        rf_info = fh.getmember(rsc_name)
        self.rsc_file = fh.extractfile(rf_info)
        print self.rsc_file
        print self.rsc_file.readline()
        print "Call to Unpack Resource"
        fh.close()

    def do_save(self, value):
        self.save_dialog.close()
        self.file_name = self.save_dialog.value['file_name'].value
        print "Save Name: " + self.output_file
    
    def do_saveas(self, value):
        self.save_dialog.close()
        self.file_name = self.save_dialog.value['file_name'].value
        print "Save As Name: " + self.output_file
        
    def add_frame_rsc(self, value):
        self.add_frame.close()
        self.file_name = self.add_frame.value['file_name'].value
        
        path, short_fn = os.path.split(self.file_name)       
        no_ext, ext = os.path.splitext(short_fn)      

        rsc_button = gui.Button(gui.Image(self.file_name, height=100))
        
        rsc_button.connect(gui.CLICK, self.edit_frame, self.position)
        self.fb_list.append((rsc_button, dict(name=no_ext, short_name=short_fn, full_name=self.file_name, ext=ext)))
        self.bt.td(rsc_button)
        self.position = self.position + 1
        
    def del_frame_rsc(self, value):
        if len(self.fb_list):
            w = self.fb_list.pop()
            self.bt.remove(w[0])
            self.position = self.position - 1
            
    def set_mode(self, mode):
        self.mode = mode
        
    def edit_frame(self, value):
        data = self.fb_list[value][1]
        self.frame_editor.edit(value)

app = App()
app.run()